import uuid

from nomic import AtlasClient
import pytest
import random
import numpy as np

def test_map_embeddings_with_errors():
    atlas = AtlasClient()

    num_embeddings = 10
    embeddings = np.random.rand(num_embeddings, 10)


    #test nested dictionaries
    with pytest.raises(Exception):
        data = [{'hello': {'hello'}} for i in range(len(embeddings))]
        response = atlas.map_embeddings(embeddings=embeddings,
                                        data=data,
                                        id_field='id',
                                        is_public=True)

    #test underscore
    with pytest.raises(Exception):
        data = [{'__hello': {'hello'} } for i in range(len(embeddings))]
        response = atlas.map_embeddings(embeddings=embeddings,
                                        data=data,
                                        id_field='id',
                                        is_public=True)

    #test non-matching keys across metadatums
    with pytest.raises(Exception):
        data = [{'hello': 'a'} for i in range(len(embeddings))]
        data[1]['goodbye'] = 'b'
        response = atlas.map_embeddings(embeddings=embeddings,
                                        data=data,
                                        id_field='id',
                                        is_public=True)

    #test duplicate keys error
    with pytest.raises(Exception):
        data = [{'id': 'a'} for i in range(len(embeddings))]
        data[1]['goodbye'] = 'b'
        response = atlas.map_embeddings(embeddings=embeddings,
                                        data=data,
                                        id_field='id',
                                        is_public=True)

def test_map_embeddings():
    atlas = AtlasClient()

    num_embeddings = 10
    embeddings = np.random.rand(num_embeddings, 10)
    data = [{'id': str(uuid.uuid4())} for i in range(len(embeddings))]

    response = atlas.map_embeddings(embeddings=embeddings,
                                    data=data,
                                    id_field='id',
                                    is_public=True)

    #progressive gives a shard upload failure for some reason
    data = [{'id': str(uuid.uuid4())} for i in range(len(embeddings))]
    atlas.update_maps(project_id=response['project_id'], data=data, embeddings=embeddings)

    assert response['project_id']
    atlas.delete_project(project_id=response['project_id'])


def test_map_text_errors():
    atlas = AtlasClient()

    # no indexed field
    with pytest.raises(Exception):
        response = atlas.map_text(data=[{'key': 'a'}],
                                  id_field='id',
                                  indexed_field='text',
                                  is_public=True,
                                  map_name='test map name',
                                  map_description='test map description',
                                  num_workers=1)

    # assert empty value for a field
    with pytest.raises(Exception):
        response = atlas.map_text(data=[{'key': ''}],
                                  id_field='id',
                                  indexed_field='key',
                                  is_public=True,
                                  map_name='test map name',
                                  map_description='test map description',
                                  num_workers=1)





import json
import os
from pathlib import Path

import click
from rich.console import Console

tenants = {
    'staging': {
        'frontend_domain': 'staging-atlas.nomic.ai',
    },
    'production': {
        'frontend_domain': 'atlas.nomic.ai',
    },
}

nomic_base_path = f'{str(Path.home())}/.nomic'


def get_api_credentials():
    if not os.path.exists(os.path.join(nomic_base_path, 'credentials')):
        raise ValueError("You have not configured your Nomic API token. Run `nomic login` to configure.")

    with open(os.path.join(nomic_base_path, 'credentials'), 'r') as file:
        credentials = json.load(file)
        return credentials


def login(token, tenant):
    environment = tenants[tenant]
    auth0_auth_endpoint = f"https://{environment['frontend_domain']}/cli-login"

    console = Console()
    style = "bold"
    if not token:
        console.print("Authorize with the Nomic API", style=style, justify="center")
        console.print(auth0_auth_endpoint, style=style, justify="center")
        console.print(
            "Click the above link to retrieve your access token and then run `nomic login \[token]`",
            style=style,
            justify="center",
        )
        exit()
    # save credential
    if not os.path.exists(nomic_base_path):
        os.mkdir(nomic_base_path)

    with open(os.path.join(nomic_base_path, 'credentials'), 'w') as file:
        json.dump({'token': token, 'tenant': tenant}, file)


@click.command()
@click.argument('command', nargs=1, default='')
@click.argument('params', nargs=-1)
def cli(command, params):
    if command == 'login':
        if len(params) == 0:
            login(token=None, tenant='production')
        if len(params) == 1 and params[0] == 'staging':
            login(token=None, tenant='staging')
        if len(params) == 2 and params[0] == 'staging':
            login(token=params[1], tenant='staging')
        if len(params) == 1:
            login(token=params[0], tenant='production')


if __name__ == "__main__":
    cli()
